unit PaintGrd;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls, Graphics, Dialogs,
  AutoGrid;

type
  TPaintBoxGrid = class(TGraphicControl)
  private
    { Private declarations }
    FMarginTop, FMarginBottom, FMarginLeft, FMarginRight,
    FXGridRange, FYGridRange, FXLabelRange, FYLabelRange: integer;
    FAGridMode: TAGridMode;
    FGridColor: TColor;
    FXMin, FXMax, FYMin, FYMax: extended;
    FXInterlabelGrid, FYInterlabelGrid, FXInteger, FYInteger: boolean;
    FOnPaint: TNotifyEvent;
    FOnXLabel, FOnYLabel: TAGLabelEvent;
  protected
    { Protected declarations }
    InPaint: boolean;
    XCoeficient, YCoeficient: extended;
    procedure SetMarginTop(V: integer);
    procedure SetMarginBottom(V: integer);
    procedure SetMarginLeft(V: integer);
    procedure SetMarginRight(V: integer);
    procedure SetXMin(V: extended);
    procedure SetXMax(V: extended);
    procedure SetYMin(V: extended);
    procedure SetYMax(V: extended);
    procedure SetXGridRange(V: integer);
    procedure SetYGridRange(V: integer);
    procedure SetXLabelRange(V: integer);
    procedure SetYLabelRange(V: integer);
    procedure SetAGridMode(GM: TAGridMode);
    procedure SetGridColor(GC: TColor);
    procedure SetXInterlabelGrid(ILG: boolean);
    procedure SetYInterlabelGrid(ILG: boolean);
    procedure SetXInteger(I: boolean);
    procedure SetYInteger(I: boolean);
    procedure Paint; override;
  public
    { Public declarations }
    property Canvas;
    constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    function GetXPoint(X: extended): integer;
    function GetYPoint(Y: extended): integer;
    procedure RepaintGrid;
  published
    { Published declarations }
    property Align;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property MarginTop: integer read FMarginTop write SetMarginTop;
    property MarginBottom: integer read FMarginBottom write SetMarginBottom;
    property MarginLeft: integer read FMarginLeft write SetMarginLeft;
    property MarginRight: integer read FMarginRight write SetMarginRight;
    property XMin: extended read FXMin write SetXMin;
    property XMax: extended read FXMax write SetXMax;
    property YMin: extended read FYMin write SetYMin;
    property YMax: extended read FYMax write SetYMax;
    property XGridRange: integer read FXGridRange write SetXGridRange;
    property YGridRange: integer read FYGridRange write SetYGridRange;
    property XLabelRange: integer read FXLabelRange write SetXLabelRange;
    property YLabelRange: integer read FYLabelRange write SetYLabelRange;
    property GridMode: TAGridMode read FAGridMode write SetAGridMode;
    property GridColor: TColor read FGridColor write SetGridColor;
    property XInterlabelGrid: boolean read FXInterlabelGrid write SetXInterlabelGrid;
    property YInterlabelGrid: boolean read FYInterlabelGrid write SetYInterlabelGrid;
    property XInteger: boolean read FXInteger write SetXInteger;
    property YInteger: boolean read FYInteger write SetYInteger;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnXLabel: TAGLabelEvent read FOnXLabel write FOnXLabel;
    property OnYLabel: TAGLabelEvent read FOnYLabel write FOnYLabel;
  end;

  TBitmapGrid = class(TComponent)
  private
    { Private declarations }
    FWidth, FHeight,
    FMarginTop, FMarginBottom, FMarginLeft, FMarginRight,
    FXGridRange, FYGridRange, FXLabelRange, FYLabelRange: integer;
    FAGridMode: TAGridMode;
    FFont: TFont;
    FColor, FGridColor: TColor;
    FXMin, FXMax, FYMin, FYMax: extended;
    FXInterlabelGrid, FYInterlabelGrid, FXInteger, FYInteger: boolean;
    FOnXLabel, FOnYLabel: TAGLabelEvent;
  protected
    { Protected declarations }
    XCoeficient, YCoeficient: extended;
    procedure SetWidth(V: integer);
    procedure SetHeight(V: integer);
    procedure SetMarginTop(V: integer);
    procedure SetMarginBottom(V: integer);
    procedure SetMarginLeft(V: integer);
    procedure SetMarginRight(V: integer);
    procedure SetXGridRange(V: integer);
    procedure SetYGridRange(V: integer);
    procedure SetXLabelRange(V: integer);
    procedure SetYLabelRange(V: integer);
  public
    { Public declarations }
    Bitmap: TBitmap;
    constructor Create(AOwner: TComponent); Override;
    Destructor Destroy; Override;
    function GetXPoint(X: extended): integer;
    function GetYPoint(Y: extended): integer;
    procedure PaintGrid;
  published
    { Published declarations }
    property Width: integer read FWidth write SetWidth;
    property Height: integer read FHeight write SetHeight;
    property MarginTop: integer read FMarginTop write SetMarginTop;
    property MarginBottom: integer read FMarginBottom write SetMarginBottom;
    property MarginLeft: integer read FMarginLeft write SetMarginLeft;
    property MarginRight: integer read FMarginRight write SetMarginRight;
    property XMin: extended read FXMin write FXMin;
    property XMax: extended read FXMax write FXMax;
    property YMin: extended read FYMin write FYMin;
    property YMax: extended read FYMax write FYMax;
    property XGridRange: integer read FXGridRange write SetXGridRange;
    property YGridRange: integer read FYGridRange write SetYGridRange;
    property XLabelRange: integer read FXLabelRange write SetXLabelRange;
    property YLabelRange: integer read FYLabelRange write SetYLabelRange;
    property GridMode: TAGridMode read FAGridMode write FAGridMode;
    property Font: TFont read FFont write FFont;
    property Color: TColor read FColor write FColor;
    property GridColor: TColor read FGridColor write FGridColor;
    property XInterlabelGrid: boolean read FXInterlabelGrid write FXInterlabelGrid;
    property YInterlabelGrid: boolean read FYInterlabelGrid write FYInterlabelGrid;
    property XInteger: boolean read FXInteger write FXInteger;
    property YInteger: boolean read FYInteger write FYInteger;
    property OnXLabel: TAGLabelEvent read FOnXLabel write FOnXLabel;
    property OnYLabel: TAGLabelEvent read FOnYLabel write FOnYLabel;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Qarsoft', [TPaintBoxGrid, TBitmapGrid]);
end;

{TPaintBoxGrid}
constructor TPaintBoxGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  InPaint := false;
  Width := 400;
  Height := 300;
  FMarginTop := 10;
  FMarginBottom := 30;
  FMarginLeft := 50;
  FMarginRight := 20;
  FXMin := -30;
  FXMax := 30;
  FYMin := 0;
  FYMax := 100;
  FXGridRange := 7;
  FYGridRange := 7;
  FXLabelRange := 20;
  FYLabelRange := 11;
  FAGridMode := mBoth;
  FGridColor := clGray;
  FXInterlabelGrid := true;
  FYInterlabelGrid := true;
  FXInteger := false;
  FYInteger := false;
  XCoeficient := 0;
  YCoeficient := 0;
end;

Destructor TPaintBoxGrid.Destroy;
begin
  inherited destroy;
end;

procedure TPaintBoxGrid.SetMarginTop(V: integer);
begin
  if V < 0 then V := 0;
  FMarginTop := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetMarginBottom(V: integer);
begin
  if V < 0 then V := 0;
  FMarginBottom := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetMarginLeft(V: integer);
begin
  if V < 0 then V := 0;
  FMarginLeft := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetMarginRight(V: integer);
begin
  if V < 0 then V := 0;
  FMarginRight := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXMin(V: extended);
begin
  FXMin := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXMax(V: extended);
begin
  FXMax := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYMin(V: extended);
begin
  FYMin := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYMax(V: extended);
begin
  FYMax := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXGridRange(V: integer);
begin
  if V < 1 then V := 1;
  FXGridRange := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYGridRange(V: integer);
begin
  if V < 1 then V := 1;
  FYGridRange := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXLabelRange(V: integer);
begin
  if V < 10 then V := 10;
  FXLabelRange := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYLabelRange(V: integer);
begin
  if V < 10 then V := 10;
  FYLabelRange := V;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetAGridMode(GM: TAGridMode);
begin
  FAGridMode := GM;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetGridColor(GC: TColor);
begin
  FGridColor := GC;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXInterlabelGrid(ILG: boolean);
begin
  FXInterlabelGrid := ILG;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYInterlabelGrid(ILG: boolean);
begin
  FYInterlabelGrid := ILG;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetXInteger(I: boolean);
begin
  FXInteger := I;
  if csDesigning in ComponentState then RepaintGrid;
end;

procedure TPaintBoxGrid.SetYInteger(I: boolean);
begin
  FYInteger := I;
  if csDesigning in ComponentState then RepaintGrid;
end;

function TPaintBoxGrid.GetXPoint(X: extended): integer;
begin
  Result := Round((X - FXMin) * XCoeficient) + FMarginLeft;
end;

function TPaintBoxGrid.GetYPoint(Y: extended): integer;
begin
  Result := Height - FMarginBottom - Round((Y - FYMin) * YCoeficient);
end;

procedure TPaintBoxGrid.RepaintGrid;
begin
  PaintAutoGrid(Self, Canvas, Width, Height,
    FMarginTop, FMarginBottom, FMarginLeft, FMarginRight,
    FXGridRange, FYGridRange, FXLabelRange, FYLabelRange,
    FAGridMode, FGridColor, FXMin, FXMax, FYMin, FYMax,
    FXInterlabelGrid, FYInterlabelGrid, FXInteger, FYInteger,
    XCoeficient, YCoeficient, Color, Font, OnXLabel, OnYLabel);
end;

procedure TPaintBoxGrid.Paint;
begin
  Inherited Paint;
  if not InPaint then begin
    InPaint := true;
    try
      RepaintGrid;
      if Assigned(OnPaint) then OnPaint(Self);
    finally
      InPaint := false;
    end;
  end;
end;

{TBitmapGrid}
constructor TBitmapGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Bitmap := TBitmap.Create;
  FFont := TFont.Create;
  FWidth := 400;
  FHeight := 300;
  FMarginTop := 10;
  FMarginBottom := 30;
  FMarginLeft := 50;
  FMarginRight := 20;
  FXMin := -30;
  FXMax := 30;
  FYMin := 0;
  FYMax := 100;
  FXGridRange := 7;
  FYGridRange := 7;
  FXLabelRange := 20;
  FYLabelRange := 13;
  FAGridMode := mBoth;
  FColor := clWhite;
  FGridColor := clGray;
  FXInterlabelGrid := true;
  FYInterlabelGrid := true;
  FXInteger := false;
  FYInteger := false;
  XCoeficient := 0;
  YCoeficient := 0;
end;

Destructor TBitmapGrid.Destroy;
begin
  BitMap.Free;
  inherited destroy;
end;

procedure TBitmapGrid.SetWidth(V: integer);
begin
  if V < 0 then V := 0;
  FWidth := V;
end;

procedure TBitmapGrid.SetHeight(V: integer);
begin
  if V < 0 then V := 0;
  FHeight := V;
end;

procedure TBitmapGrid.SetMarginTop(V: integer);
begin
  if V < 0 then V := 0;
  FMarginTop := V;
end;

procedure TBitmapGrid.SetMarginBottom(V: integer);
begin
  if V < 0 then V := 0;
  FMarginBottom := V;
end;

procedure TBitmapGrid.SetMarginLeft(V: integer);
begin
  if V < 0 then V := 0;
  FMarginLeft := V;
end;

procedure TBitmapGrid.SetMarginRight(V: integer);
begin
  if V < 0 then V := 0;
  FMarginRight := V;
end;

procedure TBitmapGrid.SetXGridRange(V: integer);
begin
  if V < 1 then V := 1;
  FXGridRange := V;
end;

procedure TBitmapGrid.SetYGridRange(V: integer);
begin
  if V < 1 then V := 1;
  FYGridRange := V;
end;

procedure TBitmapGrid.SetXLabelRange(V: integer);
begin
  if V < 10 then V := 10;
  FXLabelRange := V;
end;

procedure TBitmapGrid.SetYLabelRange(V: integer);
begin
  if V < 10 then V := 10;
  FYLabelRange := V;
end;

function TBitmapGrid.GetXPoint(X: extended): integer;
begin
  Result := Round((X - FXMin) * XCoeficient) + FMarginLeft;
end;

function TBitmapGrid.GetYPoint(Y: extended): integer;
begin
  Result := Height - FMarginBottom - Round((Y - FYMin) * YCoeficient);
end;

procedure TBitmapGrid.PaintGrid;
begin
  Bitmap.Width := Width;
  Bitmap.Height := Height;
  PaintAutoGrid(Self, Bitmap.Canvas, Width, Height,
    FMarginTop, FMarginBottom, FMarginLeft, FMarginRight,
    FXGridRange, FYGridRange, FXLabelRange, FYLabelRange,
    FAGridMode, FGridColor, FXMin, FXMax, FYMin, FYMax,
    FXInterlabelGrid, FYInterlabelGrid, FXInteger, FYInteger,
    XCoeficient, YCoeficient, FColor, FFont, OnXLabel, OnYLabel);
end;

end.

